@@ -52,7 +52,7 @@ public class GroovyFilter extends StructuredSyntaxDocumentFilter {
             "(?ms:\"{3}(?!\\\"{1,3}).*?(?:\"{3}|\\z))|(?:\"{1}(?!\\\").*?(?:\"|\\Z))";
     public static final String SINGLE_QUOTES =
             "(?ms:'{3}(?!'{1,3}).*?(?:'{3}|\\z))|(?:'[^'].*?(?:'|\\z))";
-    public static final String SLASHY_QUOTES = "/[^/*].*?/";
+    public static final String SLASHY_QUOTES = "(?:/[^/*].*?/|(?ms:\\$/.*?(?:/\\$|\\z)))";
     public static final String DIGIT = "\\d+?[efld]?";
 
     public static final String IDENT = "[\\w\\$&&[\\D]][\\w\\$]*";
