@@ -68,6 +68,7 @@ public abstract class FactoryBuilderSupport extends Binding {
     public static final String PARENT_BUILDER = "_PARENT_BUILDER_";
     public static final String CURRENT_BUILDER = "_CURRENT_BUILDER_";
     public static final String CHILD_BUILDER = "_CHILD_BUILDER_";
+    public static final String SCRIPT_CLASS_NAME = "_SCRIPT_CLASS_NAME_";
     private static final Logger LOG = Logger.getLogger(FactoryBuilderSupport.class.getName());
 
     /**
@@ -97,7 +98,7 @@ public abstract class FactoryBuilderSupport extends Binding {
                 return true;
             } else {
                 throw new RuntimeException("The value argument of '" + name + "' must be of type "
-                        + type.getName());
+                        + type.getName() + ". Found: " + value.getClass());
             }
         } else {
             return false;
@@ -121,7 +122,7 @@ public abstract class FactoryBuilderSupport extends Binding {
                 return false;
             } else {
                 throw new RuntimeException("The value argument of '" + name + "' must be of type "
-                        + type.getName() + " or a String.");
+                        + type.getName() + " or a String. Found: " + value.getClass());
             }
         } else {
             return false;
@@ -212,11 +213,6 @@ public abstract class FactoryBuilderSupport extends Binding {
         }
     }
 
-    @Deprecated
-    public FactoryBuilderSupport(Closure nameMappingClosure) {
-        this.nameMappingClosure = nameMappingClosure;
-    }
-
     /**
      * @param name the name of the variable to lookup
      * @return the variable value
@@ -656,7 +652,7 @@ public abstract class FactoryBuilderSupport extends Binding {
     }
 
     /**
-     * This method is responsible for instanciating a node and configure its
+     * This method is responsible for instantiating a node and configure its
      * properties.
      *
      * @param name       the name of the node
@@ -1119,7 +1115,17 @@ public abstract class FactoryBuilderSupport extends Binding {
         MetaClass scriptMetaClass = script.getMetaClass();
         script.setMetaClass(new FactoryInterceptorMetaClass(scriptMetaClass, this));
         script.setBinding(this);
-        return script.run();
+        Object oldScriptName = getProxyBuilder().getVariables().get(SCRIPT_CLASS_NAME);
+        try {
+            getProxyBuilder().setVariable(SCRIPT_CLASS_NAME, script.getClass().getName());
+            return script.run();
+        } finally {
+            if(oldScriptName != null) {
+                getProxyBuilder().setVariable(SCRIPT_CLASS_NAME, oldScriptName);
+            } else {
+                getProxyBuilder().getVariables().remove(SCRIPT_CLASS_NAME);
+            }
+        }
     }
 
     public Object build(final String script, GroovyClassLoader loader) {
