@@ -21,14 +21,6 @@
  * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-
-
-/*
- * GroovyScriptEngineImpl.java
- * @author Mike Grogan
- * @author A. Sundararajan
- * @author Jim White
- */
 package org.codehaus.groovy.jsr223;
 
 import groovy.lang.Binding;
@@ -58,9 +50,7 @@ import javax.script.ScriptEngineFactory;
 import javax.script.ScriptException;
 import javax.script.SimpleBindings;
 
-import java.io.ByteArrayInputStream;
 import java.io.IOException;
-import java.io.InputStream;
 import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.Writer;
@@ -73,10 +63,16 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 
+/*
+ * @author Mike Grogan
+ * @author A. Sundararajan
+ * @author Jim White
+ * @author Guillaume Laforge
+ */
 public class GroovyScriptEngineImpl
-    extends AbstractScriptEngine implements Compilable, Invocable {
+        extends AbstractScriptEngine implements Compilable, Invocable {
 
-    private static boolean DEBUG = false;
+    private static boolean debug = false;
 
     // script-string-to-generated Class map
     private Map<String, Class> classMap;
@@ -90,40 +86,42 @@ public class GroovyScriptEngineImpl
 
     // counter used to generate unique global Script class names
     private static int counter;
- 
+
     static {
         counter = 0;
     }
-    
+
     public GroovyScriptEngineImpl() {
         classMap = Collections.synchronizedMap(new HashMap<String, Class>());
         globalClosures = Collections.synchronizedMap(new HashMap<String, Closure>());
         loader = new GroovyClassLoader(getParentLoader(),
-                                       new CompilerConfiguration());
+                new CompilerConfiguration());
     }
 
-    public Object eval(Reader reader, ScriptContext ctx) 
-                       throws ScriptException {
+    public Object eval(Reader reader, ScriptContext ctx)
+            throws ScriptException {
         return eval(readFully(reader), ctx);
     }
-    
-    public Object eval(String script, ScriptContext ctx) 
-                       throws ScriptException {
+
+    public Object eval(String script, ScriptContext ctx)
+            throws ScriptException {
         try {
-            return eval(getScriptClass(script), ctx);
+            Class clazz = getScriptClass(script);
+            if (clazz == null) throw new ScriptException("Script class is null");
+            return eval(clazz, ctx);
         } catch (SyntaxException e) {
-            throw new ScriptException(e.getMessage(), 
-                                      e.getSourceLocator(), e.getLine());
+            throw new ScriptException(e.getMessage(),
+                    e.getSourceLocator(), e.getLine());
         } catch (Exception e) {
-            if (DEBUG) e.printStackTrace();
+            if (debug) e.printStackTrace();
             throw new ScriptException(e);
         }
     }
-    
+
     public Bindings createBindings() {
         return new SimpleBindings();
     }
-    
+
     public ScriptEngineFactory getFactory() {
         if (factory == null) {
             synchronized (this) {
@@ -134,49 +132,49 @@ public class GroovyScriptEngineImpl
         }
         return factory;
     }
-   
+
     // javax.script.Compilable methods 
     public CompiledScript compile(String scriptSource) throws ScriptException {
         try {
-            return new GroovyCompiledScript(this, 
-                                    getScriptClass(scriptSource));
+            return new GroovyCompiledScript(this,
+                    getScriptClass(scriptSource));
         } catch (SyntaxException e) {
-            throw new ScriptException(e.getMessage(), 
-                                      e.getSourceLocator(), e.getLine());
+            throw new ScriptException(e.getMessage(),
+                    e.getSourceLocator(), e.getLine());
         } catch (IOException e) {
             throw new ScriptException(e);
         } catch (CompilationFailedException ee) {
             throw new ScriptException(ee);
         }
-    }   
-    
+    }
+
     public CompiledScript compile(Reader reader) throws ScriptException {
         return compile(readFully(reader));
     }
-   
-    // javax.script.Invocable methods.
-    public Object invokeFunction(String name, Object... args) 
-             throws ScriptException, NoSuchMethodException  {
+
+    // javax.script.Invokable methods.
+    public Object invokeFunction(String name, Object... args)
+            throws ScriptException, NoSuchMethodException {
         return invokeImpl(null, name, args);
     }
-   
-    public Object invokeMethod(Object thiz, String name, Object... args) 
-             throws ScriptException, NoSuchMethodException  {
+
+    public Object invokeMethod(Object thiz, String name, Object... args)
+            throws ScriptException, NoSuchMethodException {
         if (thiz == null) {
             throw new IllegalArgumentException("script object is null");
         }
         return invokeImpl(thiz, name, args);
     }
-	        
-    public <T> T getInterface(Class<T> clasz) {
-        return makeInterface(null, clasz);
+
+    public <T> T getInterface(Class<T> clazz) {
+        return makeInterface(null, clazz);
     }
 
-    public <T> T getInterface(Object thiz, Class<T> clasz) {
+    public <T> T getInterface(Object thiz, Class<T> clazz) {
         if (thiz == null) {
             throw new IllegalArgumentException("script object is null");
         }
-        return makeInterface(thiz, clasz);
+        return makeInterface(thiz, clazz);
     }
 
     // package-privates
@@ -191,10 +189,10 @@ public class GroovyScriptEngineImpl
             // If we're wrapping with a PrintWriter here,
             // enable autoFlush because otherwise it might not get done!
             final Writer writer = ctx.getWriter();
-            ctx.setAttribute("out", (writer instanceof PrintWriter) ? 
-                                    writer :
-                                    new PrintWriter(writer, true),
-                                    ScriptContext.ENGINE_SCOPE);
+            ctx.setAttribute("out", (writer instanceof PrintWriter) ?
+                    writer :
+                    new PrintWriter(writer, true),
+                    ScriptContext.ENGINE_SCOPE);
 
 // Not going to do this after all (at least for now).
 // Scripts can use context.{reader, writer, errorWriter}.
@@ -217,117 +215,143 @@ public class GroovyScriptEngineImpl
 //            // always have readLine because the GDK supplies it for Reader.
 //            ctx.setAttribute("reader", ctx.getReader(), ScriptContext.ENGINE_SCOPE);
         }
-        
+
+        // Fix for GROOVY-3669: Can't use several times the same JSR-223 ScriptContext for differents groovy script
+        if (ctx.getWriter() != null) {
+            ctx.setAttribute("out", new PrintWriter(ctx.getWriter(), true), ScriptContext.ENGINE_SCOPE);
+        }
 
         /*
          * We use the following Binding instance so that global variable lookup
          * will be done in the current ScriptContext instance.
          */
         Binding binding = new Binding(ctx.getBindings(ScriptContext.ENGINE_SCOPE)) {
-                              @Override
-                              public Object getVariable(String name) {
-                                  synchronized (ctx) {
-                                      int scope = ctx.getAttributesScope(name);
-                                      if (scope != -1) {
-                                          return ctx.getAttribute(name, scope);
-                                      }
-                                  }
-                                  throw new MissingPropertyException(name, getClass());
-                              }
-                              @Override
-                              public void setVariable(String name, Object value) {
-                                  synchronized (ctx) {
-                                      int scope = ctx.getAttributesScope(name);
-                                      if (scope == -1) {    
-                                          scope = ScriptContext.ENGINE_SCOPE;
-                                      } 
-                                      ctx.setAttribute(name, value, scope);
-                                  }
-                              }
-                          };
-
-        try {
-            Script scriptObject = InvokerHelper.createScript(scriptClass, binding);
-
-            // create a Map of MethodClosures from this new script object
-            Method[] methods = scriptClass.getMethods();
-            Map<String, Closure> closures = new HashMap<String, Closure>();
-            for (Method m : methods) {
-                String name = m.getName();
-                closures.put(name, new MethodClosure(scriptObject, name));
+            @Override
+            public Object getVariable(String name) {
+                synchronized (ctx) {
+                    int scope = ctx.getAttributesScope(name);
+                    if (scope != -1) {
+                        return ctx.getAttribute(name, scope);
+                    }
+                }
+                throw new MissingPropertyException(name, getClass());
             }
 
-            // save all current closures into global closures map
-            globalClosures.putAll(closures);
+            @Override
+            public void setVariable(String name, Object value) {
+                synchronized (ctx) {
+                    int scope = ctx.getAttributesScope(name);
+                    if (scope == -1) {
+                        scope = ScriptContext.ENGINE_SCOPE;
+                    }
+                    ctx.setAttribute(name, value, scope);
+                }
+            }
+        };
 
-            MetaClass oldMetaClass = scriptObject.getMetaClass();
+        try {
+            // if this class is not an instance of Script, it's a full-blown class
+            // then simply return that class
+            if (!Script.class.isAssignableFrom(scriptClass)) {
+                return scriptClass;
+            } else {
+                // it's a script
+                Script scriptObject = (Script) scriptClass.newInstance();
+                scriptObject.setBinding(binding);
 
-            /*
-             * We override the MetaClass of this script object so that we can
-             * forward calls to global closures (of previous or future "eval" calls)
-             * This gives the illusion of working on the same "global" scope.
-             */
-            scriptObject.setMetaClass(new DelegatingMetaClass(oldMetaClass) {
-                        @Override
-                        public Object invokeMethod(Object object, String name, Object args) {
-                            if (args == null) {
-                                return invokeMethod(object, name, MetaClassHelper.EMPTY_ARRAY);
-                            }
-                            if (args instanceof Tuple) {
-                                return invokeMethod(object, name, ((Tuple)args).toArray());
-                            }
-                            if (args instanceof Object[]) {
-                                return invokeMethod(object, name, (Object[]) args);
-                            } else {
-                                return invokeMethod(object, name, new Object[] { args });
-                            }
+                // create a Map of MethodClosures from this new script object
+                Method[] methods = scriptClass.getMethods();
+                Map<String, Closure> closures = new HashMap<String, Closure>();
+                for (Method m : methods) {
+                    String name = m.getName();
+                    closures.put(name, new MethodClosure(scriptObject, name));
+                }
+
+                // save all current closures into global closures map
+                globalClosures.putAll(closures);
+
+                MetaClass oldMetaClass = scriptObject.getMetaClass();
+
+                /*
+                * We override the MetaClass of this script object so that we can
+                * forward calls to global closures (of previous or future "eval" calls)
+                * This gives the illusion of working on the same "global" scope.
+                */
+                scriptObject.setMetaClass(new DelegatingMetaClass(oldMetaClass) {
+                    @Override
+                    public Object invokeMethod(Object object, String name, Object args) {
+                        if (args == null) {
+                            return invokeMethod(object, name, MetaClassHelper.EMPTY_ARRAY);
                         }
-
-                        @Override
-                        public Object invokeMethod(Object object, String name, Object[] args) {
-                            try {
-                                return super.invokeMethod(object, name, args);
-                            } catch (MissingMethodException mme) {
-                                return callGlobal(name, args, ctx);
-                            }
+                        if (args instanceof Tuple) {
+                            return invokeMethod(object, name, ((Tuple) args).toArray());
                         }
-
-                        @Override
-                        public Object invokeStaticMethod(Object object, String name, Object[] args) {
-                            try {
-                                return super.invokeStaticMethod(object, name, args);
-                            } catch (MissingMethodException mme) {
-                                return callGlobal(name, args, ctx);
-                            }
+                        if (args instanceof Object[]) {
+                            return invokeMethod(object, name, (Object[]) args);
+                        } else {
+                            return invokeMethod(object, name, new Object[]{args});
                         }
-                    });
+                    }
 
-            return scriptObject.run();
+                    @Override
+                    public Object invokeMethod(Object object, String name, Object[] args) {
+                        try {
+                            return super.invokeMethod(object, name, args);
+                        } catch (MissingMethodException mme) {
+                            return callGlobal(name, args, ctx);
+                        }
+                    }
+
+                    @Override
+                    public Object invokeStaticMethod(Object object, String name, Object[] args) {
+                        try {
+                            return super.invokeStaticMethod(object, name, args);
+                        } catch (MissingMethodException mme) {
+                            return callGlobal(name, args, ctx);
+                        }
+                    }
+                });
+
+                return scriptObject.run();
+            }
         } catch (Exception e) {
             throw new ScriptException(e);
+        } finally {
+            // Fix for GROOVY-3669: Can't use several times the same JSR-223 ScriptContext for different groovy script
+            // Groovy's scripting engine implementation adds those two variables in the binding
+            // but should clean up afterwards
+            ctx.removeAttribute("context", ScriptContext.ENGINE_SCOPE);
+            ctx.removeAttribute("out", ScriptContext.ENGINE_SCOPE);
         }
     }
 
-    Class getScriptClass(String script) 
-                         throws SyntaxException, 
-                                CompilationFailedException, 
-                                IOException {
+    Class getScriptClass(String script)
+            throws SyntaxException,
+            CompilationFailedException,
+            IOException {
         Class clazz = classMap.get(script);
         if (clazz != null) {
             return clazz;
         }
-       
-        InputStream stream = new ByteArrayInputStream(script.getBytes()); 
-        clazz = loader.parseClass(stream, generateScriptName());
+
+        clazz = loader.parseClass(script, generateScriptName());
         classMap.put(script, clazz);
         return clazz;
     }
 
+    public void setClassLoader(final GroovyClassLoader classLoader) {
+        this.loader = classLoader;
+    }
+
+    public GroovyClassLoader getClassLoader() {
+        return this.loader;
+    }
+
     //-- Internals only below this point
 
     // invokes the specified method/function on the given object.
-    private Object invokeImpl(Object thiz, String name, Object... args) 
-             throws ScriptException, NoSuchMethodException  {
+    private Object invokeImpl(Object thiz, String name, Object... args)
+            throws ScriptException, NoSuchMethodException {
         if (name == null) {
             throw new NullPointerException("method name is null");
         }
@@ -343,7 +367,7 @@ public class GroovyScriptEngineImpl
         } catch (Exception e) {
             throw new ScriptException(e);
         }
-    } 
+    }
 
     // call the script global function of the given name
     private Object callGlobal(String name, Object[] args) {
@@ -359,10 +383,10 @@ public class GroovyScriptEngineImpl
             // given ScriptContext. If available, call it.
             Object value = ctx.getAttribute(name);
             if (value instanceof Closure) {
-                return ((Closure)value).call(args);
+                return ((Closure) value).call(args);
             } // else fall thru..
         }
-        throw new MissingMethodException(name, getClass(), args);     
+        throw new MissingMethodException(name, getClass(), args);
     }
 
     // generate a unique name for top-level Script classes
@@ -376,14 +400,14 @@ public class GroovyScriptEngineImpl
             throw new IllegalArgumentException("interface Class expected");
         }
         return (T) Proxy.newProxyInstance(
-            clazz.getClassLoader(),
-            new Class[] { clazz },
-            new InvocationHandler() {
-                public Object invoke(Object proxy, Method m, Object[] args)
-                                     throws Throwable {
-                    return invokeImpl(thiz, m.getName(), args);
-                }
-            });
+                clazz.getClassLoader(),
+                new Class[]{clazz},
+                new InvocationHandler() {
+                    public Object invoke(Object proxy, Method m, Object[] args)
+                            throws Throwable {
+                        return invokeImpl(thiz, m.getName(), args);
+                    }
+                });
     }
 
     // determine appropriate class loader to serve as parent loader
@@ -397,13 +421,14 @@ public class GroovyScriptEngineImpl
                 return ctxtLoader;
             }
         } catch (ClassNotFoundException cnfe) {
+            /* ignore */
         }
         // exception was thrown or we get wrong class
         return Script.class.getClassLoader();
     }
 
     private String readFully(Reader reader) throws ScriptException {
-        char[] arr = new char[8*1024]; // 8K at a time
+        char[] arr = new char[8 * 1024]; // 8K at a time
         StringBuilder buf = new StringBuilder();
         int numChars;
         try {
