@@ -22,13 +22,16 @@ import java.lang.reflect.Constructor;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
+import java.util.Set;
 
 import org.codehaus.groovy.reflection.CachedClass;
 import org.codehaus.groovy.reflection.ClassInfo;
 
 /**
- * Utility class for MissingMethodException, MissingProperyException etc.
+ * Utility class for MissingMethodException, MissingPropertyException etc.
  * This class contains methods assisting in ranking and listing probable intended
  * methods/fields when a exception is thrown.
  *
@@ -46,6 +49,16 @@ public class MethodRankHelper{
     public static final int MAX_METHOD_SCORE = 50;
     public static final int MAX_CONSTRUCTOR_SCORE = 20;
     public static final int MAX_FIELD_SCORE = 30;
+    
+    private static final class Pair<U,V> {
+        private U u;
+        private V v;
+        public Pair(U u, V v){
+            this.u = u;
+            this.v = v;
+        }
+    }
+    
     /**
      * Returns a string detailing possible solutions to a missing method
      * if no good solutions can be found a empty string is returned.
@@ -57,24 +70,73 @@ public class MethodRankHelper{
      */
     public static String getMethodSuggestionString(String methodName, Class type, Object[] arguments){
         ClassInfo ci = ClassInfo.getClassInfo(type);
-        List<MetaMethod> methods = new ArrayList(ci.getMetaClass().getMethods());
+        List<MetaMethod> methods = new ArrayList<MetaMethod>(ci.getMetaClass().getMethods());
         methods.addAll(ci.getMetaClass().getMetaMethods());
         List<MetaMethod> sugg = rankMethods(methodName,arguments,methods);
+        StringBuffer sb = new StringBuffer();
         if (!sugg.isEmpty()){
-            StringBuffer sb = new StringBuffer();
             sb.append("\nPossible solutions: ");
-            for(int i = 0; i < sugg.size(); i++){
+            for(int i = 0; i < sugg.size(); i++) {
                 if(i != 0) sb.append(", ");
-                sb.append(sugg.get(i).getName() + "(");
+                sb.append(sugg.get(i).getName()).append("(");
                 sb.append(listParameterNames(sugg.get(i).getParameterTypes()));
                 sb.append(")");
             }
-            return sb.toString();
-        } else{
-            return "";
         }
+        Class[] argumentClasses = getArgumentClasses(arguments);
+        List<Pair<Class,Class>> conflictClasses = getConflictClasses(sugg,argumentClasses);
+        if (!conflictClasses.isEmpty()){
+            sb.append("\nThe following classes appear as argument class and as parameter class, ");
+            sb.append("but are defined by different class loader:\n");
+            boolean first = true;
+            for(Pair<Class,Class> pair: conflictClasses) {
+                if (!first) {
+                    sb.append(", ");
+                } else {
+                    first = false;
+                }
+                sb.append(pair.u.getName()).append(" (defined by '");
+                sb.append(pair.u.getClassLoader());
+                sb.append("' and '");
+                sb.append(pair.v.getClassLoader());
+                sb.append("')");
+            }
+            sb.append("\nIf one of the method suggestions matches the method you wanted to call, ");
+            sb.append("\nthen check your class loader setup.");
+        }
+        return sb.toString();
     }
     
+    private static List<Pair<Class,Class>> getConflictClasses(List<MetaMethod> sugg, Class[] argumentClasses) {
+        List<Pair<Class,Class>> ret = new LinkedList<Pair<Class,Class>>();
+        Set<Class> recordedClasses = new HashSet<Class>();
+        for (MetaMethod method : sugg) {
+            Class[] para = method.getNativeParameterTypes();
+            for (Class aPara : para) {
+                if (recordedClasses.contains(aPara)) continue;
+                for (Class argumentClass : argumentClasses) {
+                    if (argumentClass == null) continue;
+                    if (argumentClass == aPara) continue;
+                    if (argumentClass.getName().equals(aPara.getName())) {
+                        ret.add(new Pair<Class, Class>(argumentClass, aPara));
+                    }
+                }
+                recordedClasses.add(aPara);
+            }
+        }
+        return ret;
+    }
+
+    private static Class[] getArgumentClasses(Object[] arguments) {
+        Class[] argumentClasses = new Class[arguments.length];
+        for (int i=0; i<argumentClasses.length; i++) {
+            Object arg = arguments[i];
+            if (arg==null) continue;
+            argumentClasses[i] = arg.getClass();
+        }
+        return argumentClasses;
+    }
+
     /**
      * Returns a string detailing possible solutions to a missing constructor
      * if no good solutions can be found a empty string is returned.
@@ -84,15 +146,13 @@ public class MethodRankHelper{
      * @return a string with probable solutions to the exception
      */
     public static String getConstructorSuggestionString(Class type, Object[] arguments){
-        ClassInfo ci = ClassInfo.getClassInfo(type);
-
         Constructor[] sugg = rankConstructors(arguments, type.getConstructors());
         if(sugg.length >0){
             StringBuffer sb = new StringBuffer();
             sb.append("\nPossible solutions: ");
             for(int i = 0; i < sugg.length; i++){
                 if(i != 0) sb.append(", ");
-                sb.append(type.getName() + "(");
+                sb.append(type.getName()).append("(");
                 sb.append(listParameterNames(sugg[i].getParameterTypes()));
                 sb.append(")");
             }
@@ -103,7 +163,7 @@ public class MethodRankHelper{
     }
     
     /**
-     * Returns a string detailing possible solutions to a missning field or property
+     * Returns a string detailing possible solutions to a missing field or property
      * if no good solutions can be found a empty string is returned.
      *
      * @param fieldName the missing field
@@ -113,7 +173,7 @@ public class MethodRankHelper{
     public static String getPropertySuggestionString(String fieldName, Class type){
         ClassInfo ci = ClassInfo.getClassInfo(type);
         List<MetaProperty>  fi = ci.getMetaClass().getProperties();
-        List<RankableField> rf = new ArrayList(fi.size());
+        List<RankableField> rf = new ArrayList<RankableField>(fi.size());
         StringBuffer sb = new StringBuffer();
         sb.append("\nPossible solutions: ");
         
@@ -134,6 +194,8 @@ public class MethodRankHelper{
     /**
      * creates a comma separated list of each of the class names.
      *
+     * @param cachedClasses the array of Classes
+     * @return the Class names
      */
     private static String listParameterNames(Class[] cachedClasses){
       StringBuffer sb = new StringBuffer();
@@ -155,21 +217,23 @@ public class MethodRankHelper{
       }
     
     /**
-     * Returns a sorted(ranked) list of a selection of the methods among candidates who
-     * closest reasembles original.
+     * Returns a sorted(ranked) list of a selection of the methods among candidates which
+     * most closely resembles original.
+     *
+     * @param name
      * @param original
      * @param methods
      * @return a sorted lists of Methods
      */
-    private static List<MetaMethod> rankMethods(String name, Object[] arguments, List<MetaMethod> methods) {
-        ArrayList<RankableMethod> rm = new ArrayList(methods.size());
-        if (arguments==null) arguments = new Object[0];
-        Class[] ta = new Class[arguments.length];
+    private static List<MetaMethod> rankMethods(String name, Object[] original, List<MetaMethod> methods) {
+        List<RankableMethod> rm = new ArrayList<RankableMethod>(methods.size());
+        if (original==null) original = new Object[0];
+        Class[] ta = new Class[original.length];
     
         Class nullC =  NullObject.class;
-        for(int i = 0; i < arguments.length; i++){
+        for(int i = 0; i < original.length; i++){
             //All nulls have to be wrapped so that they can be compared
-            ta[i] = arguments[i] == null?nullC: arguments[i].getClass();
+            ta[i] = original[i] == null?nullC: original[i].getClass();
         }
 
         for (MetaMethod m:methods) {
@@ -177,7 +241,7 @@ public class MethodRankHelper{
         }
         Collections.sort(rm);
         
-        List<MetaMethod> l =  new ArrayList(rm.size());
+        List<MetaMethod> l =  new ArrayList<MetaMethod>(rm.size());
         for (RankableMethod m : rm) {
             if (l.size() > MAX_RECOMENDATIONS) break;
             if (m.score > MAX_METHOD_SCORE) break;
@@ -216,33 +280,34 @@ public class MethodRankHelper{
     }
 
     /**
-     * Returns a sorted(ranked) list of a selection of the methods among candidates who
-     * closest reasembles original.
+     * Returns a sorted(ranked) list of a selection of the constructors among candidates which
+     * most closely resembles original.
+     *
      * @param original
      * @param candidates
      * @return a sorted lists of Methods
      */
-    private static Constructor[] rankConstructors(Object[] arguments, Constructor[] candidates) {
+    private static Constructor[] rankConstructors(Object[] original, Constructor[] candidates) {
         RankableConstructor[] rc = new RankableConstructor[candidates.length];
-        Class[] ta = new Class[arguments.length];
+        Class[] ta = new Class[original.length];
 
-        Class nullC =  NullObject.class;
-        for(int i = 0; i < arguments.length; i++){
+        Class nullC = NullObject.class;
+        for (int i = 0; i < original.length; i++) {
             //All nulls have to be wrapped so that they can be compared
-            ta[i] = arguments[i] == null?nullC: arguments[i].getClass();
+            ta[i] = original[i] == null ? nullC : original[i].getClass();
         }
 
-        for(int i = 0; i < candidates.length; i++){
+        for (int i = 0; i < candidates.length; i++) {
             rc[i] = new RankableConstructor(ta, candidates[i]);
         }
         Arrays.sort(rc);
-        List l = new ArrayList();
+        List<Constructor> l = new ArrayList<Constructor>();
         int index = 0;
-        while(l.size() < MAX_RECOMENDATIONS && index < rc.length && rc[index].score < MAX_CONSTRUCTOR_SCORE){
+        while (l.size() < MAX_RECOMENDATIONS && index < rc.length && rc[index].score < MAX_CONSTRUCTOR_SCORE) {
             l.add(rc[index].c);
-            index ++;
+            index++;
         }
-        return (Constructor[]) l.toArray(new Constructor[0]);
+        return l.toArray(new Constructor[l.size()]);
     }
 
     /**
@@ -262,9 +327,8 @@ public class MethodRankHelper{
                 //All args have to be boxed since argumentTypes is always boxed
                 cArgs[i] = boxVar(c.getParameterTypes()[i]);
             }
-            int argDist = damerauLevenshteinDistance(argumentTypes,cArgs);
 
-            this.score = argDist;
+            this.score = damerauLevenshteinDistance(argumentTypes,cArgs);
         }
 
         public int compareTo(Object o) {
@@ -284,8 +348,7 @@ public class MethodRankHelper{
 
         public RankableField(String name, MetaProperty mp) {
             this.f = mp;
-            int argDist = delDistance(name,mp.getName());
-            this.score = argDist;
+            this.score = delDistance(name,mp.getName());
         }
 
         public int compareTo(Object o) {
@@ -331,16 +394,16 @@ public class MethodRankHelper{
 
     /**
      * This is a slightly modified version of the Damerau Levenshtein distance
-     * algorithm. It has a additional test to see if a charackter has switched case,
+     * algorithm. It has a additional test to see if a character has switched case,
      * in the original algorithm this counts as a substitution.
      * The "cost" for a substitution is given as 10 instead of 1 in this version,
      * this enables transpositions and case modifications to have a lower cost than
      * substitutions.
      *
-     * Currently the lowercase versions of t_j and s_i isnt cached, its probable
+     * Currently the lowercase versions of t_j and s_i isn't cached, its probable
      * that some speed could be gained from this.
      * 
-     * This version is based on Chas Emerick's implementation of Lenenshtein Distance
+     * This version is based on Chas Emerick's implementation of Levenshtein Distance
      * for jakarta commons.
      * @param s a CharSequence
      * @param t the CharSequence to be compared to s
@@ -385,7 +448,7 @@ public class MethodRankHelper{
             for (i = 1; i <= n; i++) {
                 s_i = s.charAt(i - 1);
                 if (Character.isLowerCase(s_i) ^ Character.isLowerCase(t_j)) {
-                    //if s_i and t_i dont have have the same case
+                    //if s_i and t_i don't have have the same case
                     cost = caselessCompare(s_i, t_j) ? DL_CASE : DL_SUBSTITUTION;
                 } else {
                     //if they share case check for substitution
@@ -432,7 +495,7 @@ public class MethodRankHelper{
      * This is a implementation of DL distance between two Object arrays instead
      * of character streams. The objects are compared using their equals method.
      * No objects may be null.
-     * This implementation is based on Chas Emerick's implementation of Lenenshtein Distance
+     * This implementation is based on Chas Emerick's implementation of Levenshtein Distance
      * for jakarta commons.
      * @param s a Object array
      * @param t this array is compared to s
@@ -482,7 +545,6 @@ public class MethodRankHelper{
                 if(i > 1 && j > 1 && s[i -1].equals(t[j -2]) && s[i- 2].equals(t_j)){
                     vals[0][i] = Math.min(vals[0][i], vals[2][i-2] + DL_TRANSPOSITION);
                 }
-
             }
 
             // rotate all value arrays upwards(older rows get a higher index)
