@@ -95,8 +95,8 @@ public class GroovyTest extends GroovyTestCase {
      * Test that helpful "file name" appears in the stack trace and not just "Script1" 
      */
     public void testFileNameInStackTrace() {
-    	testFileNameInStackTrace("groovyErrorMsg", "\\(embedded_script_in_.*GroovyTest.xml");
-    	testFileNameInStackTrace("groovyErrorMsg_ExternalFile", "GroovyTest_errorMessage.groovy");
+        testFileNameInStackTrace("groovyErrorMsg", "\\(embedded_script_in_.*GroovyTest_dot_xml");
+        testFileNameInStackTrace("groovyErrorMsg_ExternalFile", "GroovyTest_errorMessage.groovy");
     }
 
     private void testFileNameInStackTrace(final String target, final String fileNamePattern) {
@@ -115,7 +115,7 @@ public class GroovyTest extends GroovyTestCase {
             final String stackTrace = sw.toString();
             final Pattern pattern = Pattern.compile(fileNamePattern);
             assertTrue("Does >" + stackTrace + "< contain >" + fileNamePattern + "<?", 
-            		pattern.matcher(stackTrace).find());
+                    pattern.matcher(stackTrace).find());
         }
     }
 }
